<?php defined('SYSPATH') || die('No direct script access.');

class Model_Account extends Model
{
    public static function setHashKey($key) {
        # TODO: not implemented
    }

    public static function getMaintenanceInfo() {
        $config = Model_System::loadConfig('general.json');

        $result['active'] = $config->Maintenance->active;
        $result['return'] = $config->Maintenance->return;
        return $result;
    }

    public static function with($id, $semesterID = null) {
        if ($id <= 0) throw new LogicException(Error::ID_IS_INCORRECT);

        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $sql = 'CALL `GetFullInfo`(:id, :semester);';
        return DB::query(Database::SELECT, $sql)
            ->param(':id', (int) $id)
            ->param(':semester', (int) $semesterID)
            ->execute()[0];
    }

    /**
     * @param $accountID
     * @param $newPassword
     * @return bool true, if password was changed
     */
    public static function changePassword($accountID, $newPassword) {
        $sql = 'SELECT `ChangeAccountData`(:account, :value, "password") AS Num';
        return (bool) DB::query(Database::SELECT, $sql)
            ->parameters([
                ':account' => $accountID,
                ':value'   => $newPassword,
            ])->execute()->get('Num');
    }

    /**
     * @param $accountID
     * @param $newLogin
     * @return bool true, if login was changed
     */
    public static function changeLogin($accountID, $newLogin) {
        if (!preg_match('/^[a-zA-Z0-9_\-\.]{2,}$/', $newLogin))
            throw new InvalidArgumentException('Логин содержит некорректные символы');
        
        $sql = 'SELECT `ChangeAccountData`(:account, :value, "login") AS Num';
        return (bool) DB::query(Database::SELECT, $sql)
            ->parameters([
                ':account' => $accountID,
                ':value'   => $newLogin,
            ])->execute()->get('Num');
    }

    /**
     * @param string $type 'login','email' or 'code'
     * @param string $value
     * @return bool
     */
    public static function checkAccountExistenceBy($type, $value) {
        $sql = 'SELECT `CheckAccountExistence`(:value, :type) AS Num';
        return (bool) DB::query(Database::SELECT, $sql)
            ->param(':value', $value)
            ->param(':type', $type)
            ->execute()->get('Num');
    }


    // =====================================
    //      sign up
    // =====================================

    public static function generateActivationCode() {
        $activationCode = Text::random('ABDCEFGHJKLMNPQRSTUVWXYZ123456789', 10);
        return UTF8::strtoupper($activationCode);
    }

    public static function isActivationCodeValid($code) {
        $sql = "SELECT `CheckAccountExistence`(:acode, 'code') AS Num";
        $res = DB::query(Database::SELECT, $sql)
            ->param(':acode', $code)
            ->execute()->get('Num');
        return $res == 1;
    }
    
    public static function activateAccount($login, $password, $email, $code) {
        $sql = 'SELECT `ActivateAccount`(:code, :login, :email, :pass) AS `Num`';
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':code'  => $code,
                ':login' => $login,
                ':email' => $email,
                ':pass'  => $password,
            ])->execute()->get('Num');
    }

    // =====================================
    //      authentication
    // =====================================

    public static function checkAuth($login, $password) {
        $sql = 'SELECT `SignIn`(:login, :pass) AS `ID`';
        return DB::query(Database::SELECT, $sql)
            ->param(':login', $login)
            ->param(':pass', $password)
            ->execute()->get('ID');
    }

    public static function checkAuthOpenID($globalKey) {
        $sql = 'SELECT `SignInByOpenID`(:key) AS `ID`';
        return DB::query(Database::SELECT, $sql)
            ->param(':key', $globalKey)
            ->execute()->get('ID');
    }

    public static function checkAuthToken($token) {
        $sql = 'SELECT `SignInByToken`(:token) AS `ID`';
        return DB::query(Database::SELECT, $sql)
            ->param(':token', $token)
            ->execute()->get('ID');
    }


    // =====================================
    //      password recovery
    // =====================================

    public static function createRecoveryToken($email, $token) {
        $sql = 'SELECT `CreateRecoveryToken`(:email, :token) AS UserName';
        return DB::query(Database::SELECT, $sql)
            ->param(':email', $email)
            ->param(':token', $token)
            ->execute()->get('UserName');
    }

    public static function getRecoveryInfoByEMail($email) {
        $sql = 'CALL GetRecoveryInfoByEMail(:email)';
        return DB::query(Database::SELECT, $sql)
            ->param(':email', $email)
            ->execute();
    }

    public static function getRecoveryInfoByToken($token) {
        $sql = 'CALL GetRecoveryInfoByToken(:token)';
        return DB::query(Database::SELECT, $sql)
            ->param(':token', $token)->execute();
    }

    public static function useRecoveryToken($token) {
        $sql = 'SELECT `UseRecoveryToken`(:token) AS Num';
        return DB::query(Database::SELECT, $sql)
            ->param(':token', $token)
            ->execute()->get('Num');
    }

    // =====================================
    //      auth tokens management
    // =====================================

    /**
     * @param int $accountID
     * @param int $mask bit mask with access rights
     * @return string token, if it was created. Empty string otherwise
     */
    public static function createAuthToken($accountID, $mask = 0, $description = '') {
        $sql = 'SELECT `CreateAuthToken`(:user, :description, :mask) AS `token`';
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':user' => (int) $accountID,
                ':description' => (string) $description,
                ':mask' => (int) $mask,
            ])->execute()->get('token');
    }

    public static function deleteAuthToken($token) {
        $sql = 'SELECT `deleteAuthToken`(:token) AS `res`';
        return DB::query(Database::SELECT, $sql)
            ->param(':token', $token)
            ->execute()->get('res');
    }

    /**
     * @param int $accountID - 0 to get auth tokens of all users
     * @return mixed
     */
    public static function getAuthTokens($accountID = 0) {
        $sql = 'CALL `GetAuthTokens`(:user)';
        return DB::query(Database::SELECT, $sql)
            ->param(':user', (int) $accountID)
            ->execute();
    }

}
